"""A QPlainTextEdit Widget linked to stream_loggers.

Write log messages color formatted to the given ui

# get loggers
log = get_stream_log('MyToolLog')
log_ext = get_stream_log('ExternalLog')

# create the log widget
self.loggers = QtUILogger(self, self.ui.log_layout, [log, log_ext])

log.hint('Message')

"""
import contextlib
import logging

try:
    from PySide2.QtGui import QFont
    from PySide2.QtWidgets import QPlainTextEdit
except ImportError:
    from PySide6.QtGui import QFont
    from PySide6.QtWidgets import QPlainTextEdit

from qt_log.stream_log import COLORS


class QtUILogger(logging.Handler):
    def __init__(self, parent: type, layout_widget: type, loggers: list):
        """Creates a log widget and parent the loggers to the layout widget provided.

        Args:
            parent (QtMainWindow): qt MainWindow
            layout_widget (QtLayoutWidget): the layout container for the text widget
            loggers (list): list of stream_loggers
        """
        super().__init__()

        # create output display widget, attach it to the layout widget
        self.widget = QPlainTextEdit(parent)
        layout_widget.addWidget(self.widget)

        # add loggers handlers to the widget
        self.loggers = []
        for log in loggers:
            self.loggers.append(log)
            log.addHandler(self)

        # set font, css, message format
        font = QFont('Courier New')
        self.widget.setFont(font)
        self.widget.setStyleSheet(
            """
                border: 3px solid rgb(80, 80, 80);
                color: rgb(255, 255, 255);
                background-color: rgb(20, 20, 20);
                """
        )
        self.widget.setReadOnly(True)
        self.setFormatter(logging.Formatter('%(levelname)-7s | %(message)s'))

    def close(self):
        """Removes all handlers from this widget."""
        for logger in self.loggers:
            if logger is not None:
                logger.removeHandler(self.widget)

    def emit(self, record: logging.LogRecord):
        """Writes the message formatted, uses font color based on error level number.

        Args:
            record: (logger record)
        """
        color = COLORS.get(record.levelno, 'white')
        msg = self.format(record)
        s = f"""<font color = "{color}" > {msg} </font>"""
        with contextlib.suppress(RuntimeError):
            self.widget.appendHtml(s)

    def clear(self):
        """Clears widget text."""
        self.widget.clear()



"""Custom StreamLogger.

Used for standard log in maya/hou/nuke qt tools

# create a stream logger
from stream_log import get_stream_logger
log = get_stream_log('MyToolLog')
# send messages
log.hint('Message')

"""
import logging

# Log Levels

DONE_LEVEL = logging.INFO + 1  # long operations completed
HINT_LEVEL = logging.INFO + 2  # suggestions to the user
OK_LEVEL = logging.INFO + 3  # short operation completed
PROCESS_LEVEL = logging.INFO + 4  # the beginning of a process that requires time
FILE_LEVEL = logging.INFO + 5  # filepaths or filenames

COLORS = {
    logging.DEBUG: 'yellow',
    logging.INFO: 'white',
    logging.WARNING: 'orange',
    logging.ERROR: 'red',
    logging.CRITICAL: 'purple',
    DONE_LEVEL: 'lime',
    HINT_LEVEL: 'yellow',
    OK_LEVEL: '#85c1e9',  # SKY BLUE
    PROCESS_LEVEL: '#5dade2',  # lIGHT BLUE
    FILE_LEVEL: '#aed6f1',  # SUPER LIGHT BLUE
}


def get_stream_logger(name):
    """Returns a configured custom logger."""
    log = logging.getLogger(name)
    log.setLevel(logging.DEBUG)

    # this prevent double logging in maya script editor - you can thank me later :)
    log.propagate = False

    # adding a stream handler to our logger
    stream_handler = logging.StreamHandler()
    formatter = logging.Formatter(name + ' - %(levelname)-7s | %(message)s')
    stream_handler.setFormatter(formatter)
    if len(log.handlers) == 0:
        log.addHandler(stream_handler)

    # calls
    def _done(message, *args, **kwargs):
        if log.isEnabledFor(DONE_LEVEL):
            log._log(DONE_LEVEL, message, args, **kwargs)

    def _hint(message, *args, **kwargs):
        if log.isEnabledFor(HINT_LEVEL):
            log._log(HINT_LEVEL, message, args, **kwargs)

    def _ok(message, *args, **kwargs):
        if log.isEnabledFor(OK_LEVEL):
            log._log(OK_LEVEL, message, args, **kwargs)

    def _process(message, *args, **kwargs):
        if log.isEnabledFor(PROCESS_LEVEL):
            log._log(PROCESS_LEVEL, message, args, **kwargs)

    def _file(message, *args, **kwargs):
        if log.isEnabledFor(FILE_LEVEL):
            log._log(FILE_LEVEL, message, args, **kwargs)

    # Custom Levels
    custom_levels_data = [
        (DONE_LEVEL, 'DONE', _done),
        (HINT_LEVEL, 'HINT', _hint),
        (OK_LEVEL, 'OK', _ok),
        (PROCESS_LEVEL, 'PROCESS', _process),
        (FILE_LEVEL, 'FILE', _file),
    ]

    for item in custom_levels_data:
        level, name, method = item

        # adding four steps of a custom level for python logger
        logging.addLevelName(level, name)
        setattr(log, name, level)
        setattr(logging.getLoggerClass(), name, method)
        setattr(log, name.lower(), method)

    return log


if __name__ == '__main__':
    custom_log = get_stream_logger('my_log')
    custom_log.info('test info')
    custom_log.warning('test warning')
    custom_log.error('test error')
    custom_log.critical('test critical')
    custom_log.debug('test debug')
    custom_log.ok('test ok')
    custom_log.file('test file')
    custom_log.process('test process')
    custom_log.done('test done')
    custom_log.hint('hint to the user')
